@@ -1,6 +1,6 @@
 # This file is part of the History Store (histore).
 #
-# Copyright (C) 2018-2020 New York University.
+# Copyright (C) 2018-2021 New York University.
 #
 # The History Store (histore) is released under the Revised BSD License. See
 # file LICENSE for full license details.
@@ -28,7 +28,7 @@ class ArchiveStore(metaclass=ABCMeta):
         self, schema: ArchiveSchema, writer: ArchiveWriter, version: int,
         valid_time: Optional[datetime] = None, description: Optional[str] = None,
         action: Optional[Dict] = None
-    ) -> Snapshot:  # pragma: no cover
+    ) -> Snapshot:
         """Commit a new version of the dataset archive. The modified components
         of the archive are given as the three arguments of this method.
 
@@ -55,54 +55,74 @@ class ArchiveStore(metaclass=ABCMeta):
         -------
         histore.archive.snapshot.Snapshot
         """
-        raise NotImplementedError()
+        raise NotImplementedError()  # pragma: no cover
 
     @abstractmethod
-    def is_empty(self) -> bool:  # pragma: no cover
+    def is_empty(self) -> bool:
         """True if the archive does not contain any snapshots yet.
 
         Returns
         -------
         bool
         """
-        raise NotImplementedError()
+        raise NotImplementedError()  # pragma: no cover
 
     @abstractmethod
-    def get_reader(self) -> ArchiveReader:  # pragma: no cover
+    def get_reader(self) -> ArchiveReader:
         """Get the row reader for this archive.
 
         Returns
         -------
         histore.archive.reader.ArchiveReader
         """
-        raise NotImplementedError()
+        raise NotImplementedError()  # pragma: no cover
 
     @abstractmethod
-    def get_schema(self) -> ArchiveSchema:  # pragma: no cover
+    def get_schema(self) -> ArchiveSchema:
         """Get the schema history for the archived dataset.
 
         Returns
         -------
         histore.archive.schema.ArchiveSchema
         """
-        raise NotImplementedError()
+        raise NotImplementedError()  # pragma: no cover
 
     @abstractmethod
-    def get_snapshots(self) -> SnapshotListing:  # pragma: no cover
+    def get_snapshots(self) -> SnapshotListing:
         """Get listing of all snapshots in the archive.
 
         Returns
         -------
         histore.archive.snapshot.SnapshotListing
         """
-        raise NotImplementedError()
+        raise NotImplementedError()  # pragma: no cover
 
     @abstractmethod
-    def get_writer(self) -> ArchiveWriter:  # pragma: no cover
+    def get_writer(self) -> ArchiveWriter:
         """Get a writer for a new version of the archive.
 
         Returns
         -------
         histore.archive.writer.ArchiveWriter
         """
-        raise NotImplementedError()
+        raise NotImplementedError()  # pragma: no cover
+
+    @abstractmethod
+    def rollback(self, schema: ArchiveSchema, writer: ArchiveWriter, version: int):
+        """Store the archive after it has been rolled back to a previous
+        version.
+
+        The archive schema and archive writer contain the modified schema and
+        rows after the rollback.
+
+        Parameters
+        ----------
+        schema: histore.archive.schema.ArchiveSchema
+            Schema history for the previous archive version.
+        writer: histore.archive.writer.ArchiveWriter
+            Instance of the archive writer class returned by this store that
+            was used to output the rows of the previous archive version.
+        version: int
+            Unique version identifier for the rollback snapshot.
+        """
+        raise NotImplementedError()  # pragma: no cover
