@@ -1,6 +1,6 @@
 # This file is part of the History Store (histore).
 #
-# Copyright (C) 2018-2020 New York University.
+# Copyright (C) 2018-2021 New York University.
 #
 # The History Store (histore) is released under the Revised BSD License. See
 # file LICENSE for full license details.
@@ -9,6 +9,9 @@
 a node was part of the archived document.
 """
 
+from __future__ import annotations
+from typing import List, Optional
+
 
 class TimeInterval(object):
     """Interval of integer numbers. Defines that start and end of the interval.
@@ -109,7 +112,10 @@ class Timestamp(object):
     """A timestamp is a sequence of integer numbers represented as a list of
     integer intervals.
     """
-    def __init__(self, intervals=None, version=None):
+    def __init__(
+        self, intervals: Optional[List[TimeInterval]] = None,
+        version: Optional[int] = None
+    ):
         """Initialize the timestamp using an list of time intervals. Ensures
         that intervals are not adjacent or overlapping. Raises a ValueError if
         both arguments are not None.
@@ -141,7 +147,7 @@ class Timestamp(object):
                 if self.intervals[i - 1].end + 1 >= self.intervals[i].start:
                     raise ValueError('ajacent or overlapping intervals')
 
-    def __str__(self):
+    def __str__(self) -> str:
         """Get a string representation of the timestamp.
 
         Returns
@@ -150,7 +156,7 @@ class Timestamp(object):
         """
         return ','.join([str(interval) for interval in self.intervals])
 
-    def append(self, version):
+    def append(self, version: int) -> Timestamp:
         """Append the given version to the timestamp. Returns a new modified
         timestamp. Raises ValueError if the given value is lower or equal to
         the last version in the sequence.
@@ -178,7 +184,7 @@ class Timestamp(object):
         else:
             return Timestamp(intervals=[TimeInterval(version)])
 
-    def contains(self, version):
+    def contains(self, version: int) -> bool:
         """Returns True if the timestamp contains the given value.
 
         Parameters
@@ -200,7 +206,7 @@ class Timestamp(object):
         return False
 
     @staticmethod
-    def from_string(text):
+    def from_string(text: str) -> Timestamp:
         """Create a timestamp instance from a string representation as
         generated by the __str__ method.
 
@@ -217,13 +223,13 @@ class Timestamp(object):
             pos = token.find('-')
             if pos > 0:
                 start = int(token[:pos])
-                end = int(token[pos+1:])
+                end = int(token[pos + 1:])
                 intervals.append(TimeInterval(start, end))
             else:
                 intervals.append(TimeInterval(int(token)))
         return Timestamp(intervals=intervals)
 
-    def is_empty(self):
+    def is_empty(self) -> bool:
         """Returns True if the timestamp is empty.
 
         Returns
@@ -232,7 +238,7 @@ class Timestamp(object):
         """
         return len(self.intervals) == 0
 
-    def is_equal(self, timestamp):
+    def is_equal(self, timestamp: Timestamp) -> bool:
         """Returns True if the two timestamps are equal, i.e., represent the
         same sequence of snapshots. Given that intervals cannot be consecutive,
         equality means (i) same number of intervals, and (ii) corresponding
@@ -253,7 +259,7 @@ class Timestamp(object):
             return True
         return False
 
-    def last_version(self):
+    def last_version(self) -> int:
         """Get the last version in the timestamp. If the timestamp is empty the
         result is None.
 
@@ -262,3 +268,36 @@ class Timestamp(object):
         int
         """
         return self.intervals[-1].end if self.intervals else None
+
+    def rollback(self, version: int) -> Timestamp:
+        """Remove all versions after the given version from the timestamp.
+
+        Returns a modifed timestamp.
+
+        Parameters
+        ----------
+        version: int
+            Last version in the rolled back timestamp. The result will not
+            contain version numbers that are greater than the given version.
+
+        Returns
+        -------
+        histore.archive.timestamp.Timestamp
+        """
+        # Create modified list of intervals.
+        intervals = list()
+        for interval in self.intervals:
+            if interval.start > version:
+                # Intervals are sorted. If the current interval starts after
+                # the rollback version there is nothing more to be added.
+                break
+            if interval.end <= version:
+                # Add interval since it ended before the rollback version.
+                intervals.append(interval)
+            else:
+                # The rollbacl version falls inside the interval. Clip the
+                # interval and add it to the new timestamp. All following
+                # intervals can be ignored.
+                intervals.append(TimeInterval(start=interval.start, end=version))
+                break
+        return Timestamp(intervals=intervals)
